// =========  Start of Setpoint.cpp generated code ==========
//  Setpoint.cpp from Setpoint by pfodGUIdesigner V1.0.4192
/*
  (c)2022 Forward Computing and Control Pty. Ltd.
  NSW Australia, www.forward.com.au
  This code is not warranted to be fit for any purpose. You may only use it at your own risk.
  This code may be freely used for both private and commercial use
  Provided this copyright is maintained.
*/

#ifndef SETPOINT_H
#include "Setpoint.h"
#endif

#include <SafeString.h>
Setpoint::Setpoint(pfodParser* parserPtr, pfodDwgs* dwgsPtr) : pfodDrawing(parserPtr, dwgsPtr) {
  forceReload = true; // force reload on reboot, e.g. reprogrammed
  sp = 25.0;
  errMsg[0] = '\0';
}

void Setpoint::setSP(float _sp) {
  sp = _sp;
}

float Setpoint::getSP() {
  return sp;
}


bool Setpoint::processDwgCmds() { // return true if handled else false
  byte dwgCmd = parserPtr->parseDwgCmd(); // dwgCmd is '\0' if this is NOT a dwg cmd
  if (!dwgCmd) {
    return false; // not dwg cmd not handled
  }
  if (parserPtr->dwgCmdEquals(_touchZone_cmd_6)) { // handle _touchZone_cmd_..
    Serial.print(" Got pfodAutoCmd:"); Serial.println("_touchZone_cmd_6");
    printDwgCmdReceived(&Serial); // does nothing if passed NULL
    // add your cmd handling code here
    // the input text for the sp is in parserPtr->getEditedText()
    cSFA(sfErrMsg,errMsg); // wrap errmsg
    cSFP(sfInputText,(char*)parserPtr->getEditedText()); // wrap in a SafeString
    if (!sfInputText.toFloat(sp)) {
      // error converting to float, sp is unchanged
      sfErrMsg = "Invalid setpoint : ";
      sfInputText.trim();
      sfErrMsg.readFrom(sfInputText); // put as much of the input in at will fit
    } else {
      // sp updated with new value
      // add other sp range limits 
      sfErrMsg.clear(); // no errors      
    }
    // send update
    sendUpdate();
    return true;
  }
  return false; // not handled
}


void Setpoint::drawDwg0() {
  dwgsPtr->rectangle().color(0).offset(0.00,0.00).size(7.00,4.00).centered().send();
  dwgsPtr->label().color(0).offset(-8.00,0.00).text("Temp ").fontSize(0).center().send();
  dwgsPtr->label().color(0).offset(6.00,0.00).text("\342\204\203").fontSize(0).center().send();
  dwgsPtr->index().idx(_update_idx_22).send();
  dwgsPtr->index().idx(_update_idx_23).send();
  dwgsPtr->touchZone().cmd(_touchZone_cmd_6).offset(0.00,0.00).size(7.00,4.00).centered().send();
  dwgsPtr->touchActionInput().cmd(_touchZone_cmd_6).backgroundColor(12).color(7).fontSize(6).prompt("Enter Setpoint").textIdx(_update_idx_22).send();
}

void Setpoint::updateDwg() {
  dwgsPtr->label().idx(_update_idx_22).color(0).offset(0.00,0.00).fontSize(0).center().value(sp).decimals(1).units("").send();
  dwgsPtr->label().idx(_update_idx_23).color(9).offset(0.00,4.00).text(errMsg).fontSize(0).italic().bold().center().send();
}

bool Setpoint::sendDwg() {
  if (!parserPtr->cmdEquals(*this)) {
    return false; // not this dwg's loadCmd
  }  // else
  // ======== begin debugging
  Serial.print(" sendDwg() cmd "); Serial.print((char*)parserPtr->getCmd());
  if (parserPtr->isRefresh()) {
    Serial.print(" isRefresh");
  }
  byte* argPtr = parserPtr->getFirstArg();
  if (*argPtr) {
    Serial.print("  args: ");
    while (*argPtr) {
      Serial.print("  "); Serial.print((char*)argPtr);
      argPtr = parserPtr->getNextArg(argPtr);
    }
  }
  Serial.print("\n");
  //======= end debugging =======
  if (parserPtr->isRefresh() && !forceReload) { // refresh and not forceReload just send update
    sendUpdate();
  } else {
    forceReload = false; //
    long pfodLongRtn = 0;
    uint8_t* pfodFirstArg = parserPtr->getFirstArg(); // may point to \0 if no arguments in this msg.
    parserPtr->parseLong(pfodFirstArg, &pfodLongRtn); // parse first arg as a long
    sendFullDrawing(pfodLongRtn);
  }
  return true;
}

void Setpoint::sendFullDrawing(long dwgIdx) {
  if (dwgIdx == 0) {
    dwgsPtr->start(50, 50, dwgsPtr->WHITE, true);  // the dwg size 50x50 and color (WHITE) are ignored when this drawing is inserted in the main dwg
    //true means more to come
    parserPtr->sendRefreshAndVersion(0); //need to set a version number for image refresh to work!!
    drawDwg0(); //
    dwgsPtr->end();
    
    // if drawing msg is >1023 split it in to two or more parts and add extra else if (dwgIdx == .. ) blocks here
    
  } else if (dwgIdx == 1) {
    dwgsPtr->startUpdate(false);  // always send update last so this is the last msg => false
    updateDwg(); // update with current state, // defined in display.cpp
    dwgsPtr->end();
    
  } else {
    parserPtr->print(F("{}")); // always return somthing
  }
}

void Setpoint::sendUpdate() {
  dwgsPtr->startUpdate();  // dwg refresh updates must be <1024 bytes, i.e. can only be 1 msg
  updateDwg();
  dwgsPtr->end();
}

void Setpoint::printDwgCmdReceived(Print *outPtr) {
  if (!outPtr) {
    return;
  }
  outPtr->print("     touchZone cmd "); outPtr->print((const char*)parserPtr->getDwgCmd()); outPtr->print(" at ");
  outPtr->print("("); outPtr->print(parserPtr->getTouchedX()); outPtr->print(",");
  outPtr->print(parserPtr->getTouchedY()); outPtr->print(") ");
  if (parserPtr->isTouch()) {
    outPtr->print("touch type:TOUCHED");
  }
  outPtr->println();
  if (*parserPtr->getEditedText()) { // first char not '\0'
    outPtr->print("     Edited text '"); outPtr->print((const char*)parserPtr->getEditedText()); outPtr->println("'");
  }
}
// =========  End of Setpoint.cpp generated code ==========
